@@ -58,15 +58,12 @@ class InsistentRequest(requests.Session):
                     requests.exceptions.ChunkedEncodingError):
                 continue
             return resp
-            # if 200 <= resp.status_code < 300:
-                # return resp
-            # elif 300 <= resp.status_code < 400:
-                # raise requests.HTTPError(
-                    # 'Redirect attempted with url: {}'.format(url))
-            # elif 400 <= resp.status_code < 600:
-                # continue
-        # raise requests.ConnectionError(
-            # 'Max retries exceeded with url: {}'.format(url))
+            if 200 <= resp.status_code < 300:
+                return resp
+            elif 400 <= resp.status_code < 600:
+                continue
+        raise requests.ConnectionError(
+            'Max retries exceeded with url: {}'.format(url))
 
     def get(self, url, **kwargs):
         return self.request('GET', url, **kwargs)
@@ -187,7 +184,7 @@ class Page(pyscp.core.Page):
     def source(self):
         data = self._module('viewsource/ViewSourceModule')['body']
         soup = bs4.BeautifulSoup(data, 'lxml')
-        return '\n'.join(soup.text.split('\n')[1:])
+        return ''.join([s + '\n' for s in soup.text.replace('\t','').split('\n')[1:]][1:])
 
     @property
     def created(self):
@@ -213,10 +210,11 @@ class Page(pyscp.core.Page):
         parsed = []
         for file in files:
             url = self._wiki.site + file.find('a')['href']
+            file_id = int(re.search('event, ([0-9]+)', str(file)).group(1))
             name = file.find('a').text.strip()
             filetype = file('td')[1].text.strip()
             size = file('td')[2].text.strip()
-            parsed.append(pyscp.core.File(url, name, filetype, size))
+            parsed.append(pyscp.core.File(url, file_id, name, filetype, size))
         return parsed
 
     ###########################################################################
@@ -259,7 +257,7 @@ class Page(pyscp.core.Page):
 
     def set_tags(self, tags):
         """Replace the tags of the page."""
-        res = self._action('saveTags', tags=' '.join(tags))
+        res = self._action('saveTags', tags=''.join(tags))
         self._flush('history', '_pdata')
         return res
 
@@ -275,13 +273,36 @@ class Page(pyscp.core.Page):
             url,
             data=kwargs,
             files={'userfile': (name, data)},
-            cookies={'wikidot_token7': '7fbc760204fb4082aa5e47b274d47f54'})
+            headers={'Cookie': self._wiki.cookies})
         response = bs4.BeautifulSoup(response.text, 'lxml')
         status = response.find(id='status').text
         message = response.find(id='message').text
-        if status != 'ok':
+        if status != 'ok' and status != 'file_exists':
             raise RuntimeError(message)
+        elif status == 'file_exists':
+            self.remove_file(name)
+            response = self._wiki.req.post(
+                url,
+                data=kwargs,
+                files={'userfile': (name, data)},
+                headers={'Cookie': self._wiki.cookies})
+            response = bs4.BeautifulSoup(response.text, 'lxml')
+            status = response.find(id='status').text
+            message = response.find(id='message').text
+            if status != 'ok':
+                raise RuntimeError(message)
         return response
+    
+    def remove_file(self,name):
+        files = self.files
+        id = -1
+        for file in files:
+            if file.name == name:
+                id = file.id
+        self._module('Empty',
+            event='deleteFile',
+            action='FileAction',
+            file_id=id)
 
     ###########################################################################
     # Voting Methods
@@ -512,6 +533,16 @@ class Wiki(pyscp.core.Wiki):
                 event='send',
                 wikidot_token7='123456'),
             headers={'Content-Type': 'application/x-www-form-urlencoded;', 'Cookie':  self.cookies}).json()
+    
+    ###########################################################################
+    # Properties
+    ###########################################################################
+    
+    @property
+    def title(self):
+        data = self.req.get(self.site).text
+        soup = bs4.BeautifulSoup(data, 'lxml')
+        return soup.title.string
 
     ###########################################################################
     # SCP-Wiki Specific Methods
@@ -578,35 +609,37 @@ class User(pyscp.core.User):
     def _pdata(self):
         data = self.req.get(self.url).text
         soup = bs4.BeautifulSoup(data, 'lxml')
-        return (int(re.search('userId = ([0-9]+);', data).group(1)),
-                self.parse_data(soup))
+        return self.parse_data(soup)
     
     @pyscp.utils.log_errors(log.warning)
     def parse_data(self, soup):
         content = soup.find(id='page-content')
         time = parse_element_time(content)
-        data = soup.find('div', {'id': 'user-info-area'}).find_all('dd')
-        new_data = []
-        for i in data:
-            new_data.append(i.get_text())
-        if 'none' in new_data[2]:
+        level = 'none'
+        if 'none' in content:
             level = 'none'
-        elif 'low' in new_data[2]:
+        elif 'low' in content:
             level = 'low'
-        elif 'medium' in new_data[2]:
+        elif 'medium' in content:
             level = 'medium'
-        elif 'high' in new_data[2] and 'very' not in new_data[2]:
+        elif 'high' in content and 'very' not in content:
             level = 'high'
-        elif 'very high' in new_data[2]:
+        elif 'very high' in content:
             level = 'very high'
-        elif 'guru' in new_data[2]:
+        elif 'guru' in content:
             level = 'guru'
-        parsed_data = {'name': self.username, 'time': time, 'type': new_data[1].replace('\n','').replace(' ',''), 'level': level}
-        return parsed_data
+        utype = 'free'
+        if 'free' in content:
+            utype = 'free'
+        elif 'Pro' in content:
+            utype = 'Pro'
+        return {'name': self.username, 'time': time, 'type': utype, 'level': level}
     
     @property
     def id(self):
-        return self._pdata[0]
+        data = self.req.get(self.url).text
+        soup = bs4.BeautifulSoup(data, 'lxml')
+        return int(re.search('userId = ([0-9]+);', data).group(1))
     
     ###########################################################################
     # Properties
@@ -614,15 +647,15 @@ class User(pyscp.core.User):
     
     @property
     def join_time(self):
-        return self._pdata[1]['time']
+        return self._pdata['time']
     
     @property
     def type(self):
-        return self._pdata[1]['type']
+        return self._pdata['type']
     
     @property
     def karma(self):
-        return self._pdata[1]['level']
+        return self._pdata['level']
     
     @property
     def member(self):
